using System;
using System.Collections;
using UnityEngine;
using UnityEngine.Events;

namespace PropMod
{
    public class PropMod
    {
        public static string tag = " -PM";
        public static void Main()
        {
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Plate"),
                    NameOverride = "Cup" +tag,
                    NameToOrderByOverride = "!",
                    DescriptionOverride = "Destructible cup.",
                    CategoryOverride = ModAPI.FindCategory("Misc."),
                    ThumbnailOverride = ModAPI.LoadSprite("Textures/CupThumb.png"),
                    AfterSpawn = (Instance) =>
                    {
                        GameObject.Destroy(Instance.GetComponent<DestroyableBehaviour>());

                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Textures/Cup.png");

                        var destructibleObjectBehaviour = Instance.GetOrAddComponent<DestructibleObjectBehaviour>();
                        destructibleObjectBehaviour.breakSounds = new AudioClip[]
                        {
                            ModAPI.LoadSound("Sounds/CupBreak1.mp3"),
                            ModAPI.LoadSound("Sounds/CupBreak2.mp3"),
                            ModAPI.LoadSound("Sounds/CupBreak3.mp3")
                        };
                        destructibleObjectBehaviour.debrisSprites = new Sprite[]
                        {
                            ModAPI.LoadSprite("Textures/CupDebris1.png"),
                            ModAPI.LoadSprite("Textures/CupDebris2.png")
                        };
                        destructibleObjectBehaviour.debrisPositions = new Vector3[]
                        {
                            new Vector3(0f, 0f),
                            new Vector3(0f, 0f)
                        };
                        destructibleObjectBehaviour.debrisProperties = ModAPI.FindPhysicalProperties("Glass");
                        destructibleObjectBehaviour.debrisMass = 0.0045f;
                        destructibleObjectBehaviour.BreakingThreshold = 0.03f;
                        destructibleObjectBehaviour.debrisNumber = 2;

                        Instance.FixColliders();
                    }
                }
            );
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Plate"),
                    NameOverride = "Glass Bottle" +tag,
                    NameToOrderByOverride = "!",
                    DescriptionOverride = "Destructible bottle.",
                    CategoryOverride = ModAPI.FindCategory("Misc."),
                    ThumbnailOverride = ModAPI.LoadSprite("Textures/BottleThumb.png"),
                    AfterSpawn = (Instance) =>
                    {
                        GameObject.Destroy(Instance.GetComponent<DestroyableBehaviour>());

                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Textures/Bottle1.png");

                        Instance.GetOrAddComponent<RandomSpriteBehaviour>().sprites = new Sprite[]
                        {
                            ModAPI.LoadSprite("Textures/Bottle1.png"),
                            ModAPI.LoadSprite("Textures/Bottle2.png"),
                            ModAPI.LoadSprite("Textures/Bottle3.png")
                        };

                        var phys = Instance.GetComponent<PhysicalBehaviour>();
                        phys.TrueInitialMass = 0.02f;
                        phys.InitialMass = 0.02f;
                        phys.rigidbody.mass = 0.02f;

                        var destructibleObjectBehaviour = Instance.GetOrAddComponent<DestructibleObjectBehaviour>();
                        destructibleObjectBehaviour.breakSounds = new AudioClip[]
                        {
                            ModAPI.LoadSound("Sounds/BottleBreak1.mp3"),
                            ModAPI.LoadSound("Sounds/BottleBreak2.mp3"),
                            ModAPI.LoadSound("Sounds/BottleBreak3.mp3")
                        };
                        destructibleObjectBehaviour.debrisPositions = new Vector3[]
                        {
                            new Vector3(0.00016f, -0.17146f),
                            new Vector3(0.0001654625f, -0.05720842f),
                            new Vector3(-0.01432f, 0.11427f)
                        };
                        destructibleObjectBehaviour.debrisProperties = ModAPI.FindPhysicalProperties("Glass");
                        destructibleObjectBehaviour.debrisMass = 0.006f;
                        destructibleObjectBehaviour.BreakingThreshold = 0.06f;
                        destructibleObjectBehaviour.debrisNumber = 3;

                        var bottleBehaviour = Instance.GetOrAddComponent<BottleBehaviour>();
                        bottleBehaviour.sprites1 = new Sprite[]
                        {
                            ModAPI.LoadSprite("Textures/Bottle1Debris1.png"),
                            ModAPI.LoadSprite("Textures/Bottle1Debris2.png"),
                            ModAPI.LoadSprite("Textures/Bottle1Debris3.png")
                        };
                        bottleBehaviour.sprites2 = new Sprite[]
                        {
                            ModAPI.LoadSprite("Textures/Bottle2Debris1.png"),
                            ModAPI.LoadSprite("Textures/Bottle2Debris2.png"),
                            ModAPI.LoadSprite("Textures/Bottle2Debris3.png")
                        };
                        bottleBehaviour.sprites3 = new Sprite[]
                        {
                            ModAPI.LoadSprite("Textures/Bottle3Debris1.png"),
                            ModAPI.LoadSprite("Textures/Bottle3Debris2.png"),
                            ModAPI.LoadSprite("Textures/Bottle3Debris3.png")
                        };

                        Instance.FixColliders();
                    }
                }
            );
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Stick"),
                    NameOverride = "Cardboard Box" +tag,
                    NameToOrderByOverride = "!",
                    DescriptionOverride = "Regular cardboard box.",
                    CategoryOverride = ModAPI.FindCategory("Misc."),
                    ThumbnailOverride = ModAPI.LoadSprite("Textures/CardboardBoxThumb.png"),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Textures/CardboardBox.png");

                        var phys = Instance.GetComponent<PhysicalBehaviour>();
                        phys.TrueInitialMass = 0.35f;
                        phys.InitialMass = 0.35f;
                        phys.rigidbody.mass = 0.35f;
                        var properties = ModAPI.FindPhysicalProperties("Wood");
                        properties.name = "Cardboard";
                        properties.Buoyancy = 1f;
                        properties.Flammability = 100f;
                        properties.Brittleness = 50f;
                        properties.BulletSpeedAbsorptionPower = 0.01f;
                        properties.Burnrate = 1f;
                        properties.Softness = 0.9f;
                        properties.SoftImpact = new AudioClip[]
                        {
                            ModAPI.LoadSound("Sounds/Cardboard1.mp3"),
                            ModAPI.LoadSound("Sounds/Cardboard2.mp3")
                        };
                        properties.HardImpact = new AudioClip[]
                        {
                            ModAPI.LoadSound("Sounds/Cardboard1.mp3"),
                            ModAPI.LoadSound("Sounds/Cardboard2.mp3")
                        };
                        properties.StabSound = new AudioClip[]
                        {
                            ModAPI.LoadSound("Sounds/Cardboard1.mp3"),
                            ModAPI.LoadSound("Sounds/Cardboard2.mp3")
                        };
                        phys.Properties = properties;

                        Instance.FixColliders();
                    }
                }
            );
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Stick"),
                    NameOverride = "Big Cardboard Box" +tag,
                    NameToOrderByOverride = "!",
                    DescriptionOverride = "Big cardboard box.",
                    CategoryOverride = ModAPI.FindCategory("Misc."),
                    ThumbnailOverride = ModAPI.LoadSprite("Textures/BigCardboardBoxThumb.png"),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Textures/BigCardboardBox.png");

                        var phys = Instance.GetComponent<PhysicalBehaviour>();
                        phys.TrueInitialMass = 0.6f;
                        phys.InitialMass = 0.6f;
                        phys.rigidbody.mass = 0.6f;
                        var properties = ModAPI.FindPhysicalProperties("Wood");
                        properties.name = "Cardboard";
                        properties.Buoyancy = 1f;
                        properties.Flammability = 100f;
                        properties.Brittleness = 50f;
                        properties.BulletSpeedAbsorptionPower = 0.01f;
                        properties.Burnrate = 1f;
                        properties.Softness = 0.9f;
                        properties.SoftImpact = new AudioClip[]
                        {
                            ModAPI.LoadSound("Sounds/Cardboard1.mp3"),
                            ModAPI.LoadSound("Sounds/Cardboard2.mp3")
                        };
                        properties.HardImpact = new AudioClip[]
                        {
                            ModAPI.LoadSound("Sounds/Cardboard1.mp3"),
                            ModAPI.LoadSound("Sounds/Cardboard2.mp3")
                        };
                        properties.StabSound = new AudioClip[]
                        {
                            ModAPI.LoadSound("Sounds/Cardboard1.mp3"),
                            ModAPI.LoadSound("Sounds/Cardboard2.mp3")
                        };
                        phys.Properties = properties;

                        Instance.FixColliders();
                    }
                }
            );
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Bulletproof Sheet"),
                    NameOverride = "Safe" +tag,
                    NameToOrderByOverride = "!",
                    DescriptionOverride = "Standart safe.",
                    CategoryOverride = ModAPI.FindCategory("Misc."),
                    ThumbnailOverride = ModAPI.LoadSprite("Textures/SafeThumb.png"),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground";

                        var phys = Instance.GetComponent<PhysicalBehaviour>();
                        phys.TrueInitialMass = 6f;
                        phys.InitialMass = 6f;
                        phys.rigidbody.mass = 6f;

                        var collider1 = Instance.GetComponent<BoxCollider2D>();
                        collider1.offset = new Vector3(-0.2999443f, 0f);
                        collider1.size = new Vector3(0.1140525f, 0.9714286f);
                        var collider2 = Instance.AddComponent<BoxCollider2D>();
                        collider2.offset = new Vector3(0.01393867f, 0.4285474f);
                        collider2.size = new Vector3(0.6291509f, 0.1143538f);
                        var collider3 = Instance.AddComponent<BoxCollider2D>();
                        collider3.offset = new Vector3(0.01534551f, -0.4287081f);
                        collider3.size = new Vector3(0.6263442f, 0.1144476f);
                        var collider4 = Instance.AddComponent<BoxCollider2D>();
                        collider4.offset = new Vector3(0.2995926f, -0.001843572f);
                        collider4.size = new Vector3(0.05785f, 0.8573329f);

                        var safeBehaviour = Instance.GetOrAddComponent<SafeBehaviour>();
                        safeBehaviour.Collider = collider4;
                        safeBehaviour.OpenSound = ModAPI.LoadSound("Sounds/Safe1.mp3");
                        safeBehaviour.ClosedSound = ModAPI.LoadSound("Sounds/Safe2.mp3");
                        var safeSprite1 = new SafeBehaviour.SafeSprites();
                        safeSprite1.OpenSprite = ModAPI.LoadSprite("Textures/Safe1Open.png");
                        safeSprite1.ClosedSprite = ModAPI.LoadSprite("Textures/Safe1Closed.png");
                        var safeSprite2 = new SafeBehaviour.SafeSprites();
                        safeSprite2.OpenSprite = ModAPI.LoadSprite("Textures/Safe2Open.png");
                        safeSprite2.ClosedSprite = ModAPI.LoadSprite("Textures/Safe2Closed.png");
                        var safeSprite3 = new SafeBehaviour.SafeSprites();
                        safeSprite3.OpenSprite = ModAPI.LoadSprite("Textures/Safe3Open.png");
                        safeSprite3.ClosedSprite = ModAPI.LoadSprite("Textures/Safe3Closed.png");
                        safeBehaviour.Textures = new SafeBehaviour.SafeSprites[]
                        {
                            safeSprite1,
                            safeSprite2,
                            safeSprite3
                        };
                    }
                }
            );
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Bulletproof Sheet"),
                    NameOverride = "Big Safe" +tag,
                    NameToOrderByOverride = "!",
                    DescriptionOverride = "Big safe made of bulletproof material.",
                    CategoryOverride = ModAPI.FindCategory("Misc."),
                    ThumbnailOverride = ModAPI.LoadSprite("Textures/BigSafeThumb.png"),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sortingLayerName = "Foreground";

                        var phys = Instance.GetComponent<PhysicalBehaviour>();
                        phys.TrueInitialMass = 15f;
                        phys.InitialMass = 15f;
                        phys.rigidbody.mass = 15f;

                        var collider1 = Instance.GetComponent<BoxCollider2D>();
                        collider1.offset = new Vector3(-0.4143625f, 0f);
                        collider1.size = new Vector3(0.1143271f, 1.828571f);
                        var collider2 = Instance.AddComponent<BoxCollider2D>();
                        collider2.offset = new Vector3(0.008222938f, -0.8572956f);
                        collider2.size = new Vector3(0.8691599f, 0.1144679f);
                        var collider3 = Instance.AddComponent<BoxCollider2D>();
                        collider3.offset = new Vector3(0.008222938f, 0.857052f);
                        collider3.size = new Vector3(0.8691599f, 0.1144679f);
                        var collider4 = Instance.AddComponent<BoxCollider2D>();
                        collider4.offset = new Vector3(0.4021896f, -0.003237367f);
                        collider4.size = new Vector3(0.08122665f, 1.684231f);

                        var safeBehaviour = Instance.GetOrAddComponent<SafeBehaviour>();
                        safeBehaviour.Collider = collider4;
                        safeBehaviour.OpenSound = ModAPI.LoadSound("Sounds/BigSafe1.mp3");
                        safeBehaviour.ClosedSound = ModAPI.LoadSound("Sounds/BigSafe2.mp3");
                        var safeSprite1 = new SafeBehaviour.SafeSprites();
                        safeSprite1.OpenSprite = ModAPI.LoadSprite("Textures/BigSafe1Open.png");
                        safeSprite1.ClosedSprite = ModAPI.LoadSprite("Textures/BigSafe1Closed.png");
                        var safeSprite2 = new SafeBehaviour.SafeSprites();
                        safeSprite2.OpenSprite = ModAPI.LoadSprite("Textures/BigSafe2Open.png");
                        safeSprite2.ClosedSprite = ModAPI.LoadSprite("Textures/BigSafe2Closed.png");
                        var safeSprite3 = new SafeBehaviour.SafeSprites();
                        safeSprite3.OpenSprite = ModAPI.LoadSprite("Textures/BigSafe3Open.png");
                        safeSprite3.ClosedSprite = ModAPI.LoadSprite("Textures/BigSafe3Closed.png");
                        safeBehaviour.Textures = new SafeBehaviour.SafeSprites[]
                        {
                            safeSprite1,
                            safeSprite2,
                            safeSprite3
                        };
                    }
                }
            );
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Stick"),
                    NameOverride = "Money" +tag,
                    NameToOrderByOverride = "!",
                    DescriptionOverride = "A money.",
                    CategoryOverride = ModAPI.FindCategory("Misc."),
                    ThumbnailOverride = ModAPI.LoadSprite("Textures/MoneyThumb.png"),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Textures/Money.png");

                        var phys = Instance.GetComponent<PhysicalBehaviour>();
                        phys.TrueInitialMass = 0.003f;
                        phys.InitialMass = 0.003f;
                        phys.rigidbody.mass = 0.003f;
                        var properties = ModAPI.FindPhysicalProperties("Bowling pin");
                        properties.name = "Paper";
                        properties.Buoyancy = 1f;
                        properties.Flammability = 100f;
                        properties.Brittleness = 100f;
                        properties.BulletSpeedAbsorptionPower = 0.005f;
                        properties.Burnrate = 1f;
                        properties.Softness = 0.95f;
                        properties.SoftImpact = new AudioClip[]
                        {
                            ModAPI.LoadSound("Sounds/Paper1.mp3"),
                            ModAPI.LoadSound("Sounds/Paper2.mp3"),
                            ModAPI.LoadSound("Sounds/Paper3.mp3")
                        };
                        properties.HardImpact = new AudioClip[]
                        {
                            ModAPI.LoadSound("Sounds/Paper1.mp3"),
                            ModAPI.LoadSound("Sounds/Paper2.mp3"),
                            ModAPI.LoadSound("Sounds/Paper3.mp3")
                        };
                        properties.StabSound = new AudioClip[]
                        {
                            ModAPI.LoadSound("Sounds/Paper1.mp3"),
                            ModAPI.LoadSound("Sounds/Paper2.mp3"),
                            ModAPI.LoadSound("Sounds/Paper3.mp3")
                        };
                        phys.Properties = properties;

                        Instance.FixColliders();
                    }
                }
            );
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Metal Pole"),
                    NameOverride = "Oil Barrel" +tag,
                    NameToOrderByOverride = "!",
                    DescriptionOverride = "Barrel with oil.",
                    CategoryOverride = ModAPI.FindCategory("Misc."),
                    ThumbnailOverride = ModAPI.LoadSprite("Textures/OilBarrelThumb.png"),
                    AfterSpawn = (Instance) =>
                    {
                        GameObject.Destroy(Instance.GetComponent<GlowingHotMetalBehaviour>());

                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Textures/OilBarrel.png");

                        var phys = Instance.GetComponent<PhysicalBehaviour>();
                        phys.TrueInitialMass = 1f;
                        phys.InitialMass = 1f;
                        phys.rigidbody.mass = 1f;
                        phys.Properties = ModAPI.FindPhysicalProperties("Flammable metal");

                        Instance.FixColliders();
                    }
                }
            );
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Metal Pole"),
                    NameOverride = "Water Barrel" +tag,
                    NameToOrderByOverride = "!",
                    DescriptionOverride = "Barrel with fresh water.",
                    CategoryOverride = ModAPI.FindCategory("Misc."),
                    ThumbnailOverride = ModAPI.LoadSprite("Textures/WaterBarrelThumb.png"),
                    AfterSpawn = (Instance) =>
                    {
                        GameObject.Destroy(Instance.GetComponent<GlowingHotMetalBehaviour>());

                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Textures/WaterBarrel.png");

                        var phys = Instance.GetComponent<PhysicalBehaviour>();
                        phys.TrueInitialMass = 1f;
                        phys.InitialMass = 1f;
                        phys.rigidbody.mass = 1f;
                        phys.Properties = ModAPI.FindPhysicalProperties("Flammable metal");

                        Instance.FixColliders();
                    }
                }
            );
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Metal Pole"),
                    NameOverride = "Hazard Barrel" +tag,
                    NameToOrderByOverride = "!",
                    DescriptionOverride = "Barrel with radioactive waste.",
                    CategoryOverride = ModAPI.FindCategory("Misc."),
                    ThumbnailOverride = ModAPI.LoadSprite("Textures/HazardBarrelThumb.png"),
                    AfterSpawn = (Instance) =>
                    {
                        GameObject.Destroy(Instance.GetComponent<GlowingHotMetalBehaviour>());

                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Textures/HazardBarrel.png");

                        var phys = Instance.GetComponent<PhysicalBehaviour>();
                        phys.TrueInitialMass = 1f;
                        phys.InitialMass = 1f;
                        phys.rigidbody.mass = 1f;
                        phys.Properties = ModAPI.FindPhysicalProperties("Flammable metal");

                        Instance.FixColliders();
                    }
                }
            );
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Metal Pole"),
                    NameOverride = "Counting Pedestal" +tag,
                    NameToOrderByOverride = "!",
                    DescriptionOverride = "A counting pedestal.",
                    CategoryOverride = ModAPI.FindCategory("Misc."),
                    ThumbnailOverride = ModAPI.LoadSprite("Textures/CountingPedestalThumb.png"),
                    AfterSpawn = (Instance) =>
                    {
                        GameObject.Destroy(Instance.GetComponent<GlowingHotMetalBehaviour>());

                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Textures/CountingPedestal.png");

                        var collider1 = Instance.GetComponent<BoxCollider2D>();
                        collider1.offset = new Vector3(0f, -0.41446f);
                        collider1.size = new Vector3(2.514286f, 0.8567964f);
                        var collider2 = Instance.AddComponent<BoxCollider2D>();
                        collider2.offset = new Vector3(0.0001291931f, 0f);
                        collider2.size = new Vector3(0.856757f, 1.685714f);

                        var phys = Instance.GetComponent<PhysicalBehaviour>();
                        phys.TrueInitialMass = 9f;
                        phys.InitialMass = 9f;
                        phys.rigidbody.mass = 9f;
                        phys.Properties = ModAPI.FindPhysicalProperties("Flammable metal");
                    }
                }
            );
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Rod"),
                    NameOverride = "Clock" +tag,
                    NameToOrderByOverride = "!",
                    DescriptionOverride = "Shows the real time.",
                    CategoryOverride = ModAPI.FindCategory("Misc."),
                    ThumbnailOverride = ModAPI.LoadSprite("Textures/ClockThumb.png"),
                    AfterSpawn = (Instance) =>
                    {
                        GameObject.Destroy(Instance.GetComponent<GlowingHotMetalBehaviour>());

                        var renderer = Instance.GetComponent<SpriteRenderer>();
                        renderer.sprite = ModAPI.LoadSprite("Textures/Clock.png");

                        var phys = Instance.GetComponent<PhysicalBehaviour>();
                        phys.TrueInitialMass = 0.1f;
                        phys.InitialMass = 0.1f;
                        phys.rigidbody.mass = 0.1f;

                        var secondArrow = new GameObject("Second Arrow");
                        secondArrow.transform.parent = Instance.transform;
                        secondArrow.transform.localPosition = Vector3.zero;
                        var secondArrowRenderer = secondArrow.GetOrAddComponent<SpriteRenderer>();
                        secondArrowRenderer.sprite = ModAPI.LoadSprite("Textures/ClockSecondArrow.png");
                        secondArrowRenderer.sortingLayerName = renderer.sortingLayerName;
                        secondArrowRenderer.sortingOrder = renderer.sortingOrder + 3;

                        var minuteArrow = new GameObject("Minute Arrow");
                        minuteArrow.transform.parent = Instance.transform;
                        minuteArrow.transform.localPosition = Vector3.zero;
                        var minuteArrowRenderer = minuteArrow.GetOrAddComponent<SpriteRenderer>();
                        minuteArrowRenderer.sprite = ModAPI.LoadSprite("Textures/ClockMinuteArrow.png");
                        minuteArrowRenderer.sortingLayerName = renderer.sortingLayerName;
                        minuteArrowRenderer.sortingOrder = renderer.sortingOrder + 2;

                        var hourArrow = new GameObject("Hour Arrow");
                        hourArrow.transform.parent = Instance.transform;
                        hourArrow.transform.localPosition = Vector3.zero;
                        var hourArrowRenderer = hourArrow.GetOrAddComponent<SpriteRenderer>();
                        hourArrowRenderer.sprite = ModAPI.LoadSprite("Textures/ClockHourArrow.png");
                        hourArrowRenderer.sortingLayerName = renderer.sortingLayerName;
                        hourArrowRenderer.sortingOrder = renderer.sortingOrder + 1;

                        var clockBehaviour = Instance.GetOrAddComponent<ClockBehaviour>();
                        clockBehaviour.secondArrow = secondArrow.transform;
                        clockBehaviour.minuteArrow = minuteArrow.transform;
                        clockBehaviour.hourArrow = hourArrow.transform;
                        clockBehaviour.tick = ModAPI.LoadSound("Sounds/Clock.mp3");

                        Instance.transform.localScale = new Vector3(1f, 1f);
                        Instance.FixColliders();
                    }
                }
            );
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Rod"),
                    NameOverride = "Gold Bar" +tag,
                    NameToOrderByOverride = "!",
                    DescriptionOverride = "Bar made of gold.",
                    CategoryOverride = ModAPI.FindCategory("Misc."),
                    ThumbnailOverride = ModAPI.LoadSprite("Textures/GoldBarThumb.png"),
                    AfterSpawn = (Instance) =>
                    {
                        GameObject.Destroy(Instance.GetComponent<GlowingHotMetalBehaviour>());

                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Textures/GoldBar.png");

                        var phys = Instance.GetComponent<PhysicalBehaviour>();
                        phys.TrueInitialMass = 0.045f;
                        phys.InitialMass = 0.045f;
                        phys.rigidbody.mass = 0.045f;
                        var properties = ModAPI.FindPhysicalProperties("Flammable metal");
                        properties.BulletSpeedAbsorptionPower = 0.4f;
                        properties.Brittleness = 0.2f;
                        phys.Properties = properties;

                        Instance.FixColliders();
                    }
                }
            );
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Plate"),
                    NameOverride = "Precious Stone" +tag,
                    NameToOrderByOverride = "!",
                    DescriptionOverride = "A precious stone.",
                    CategoryOverride = ModAPI.FindCategory("Misc."),
                    ThumbnailOverride = ModAPI.LoadSprite("Textures/PreciousStoneThumb.png"),
                    AfterSpawn = (Instance) =>
                    {
                        GameObject.Destroy(Instance.GetComponent<DestroyableBehaviour>());

                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Textures/PreciousStone1.png");

                        Instance.GetOrAddComponent<RandomSpriteBehaviour>().sprites = new Sprite[]
                        {
                            ModAPI.LoadSprite("Textures/PreciousStone1.png"),
                            ModAPI.LoadSprite("Textures/PreciousStone2.png"),
                            ModAPI.LoadSprite("Textures/PreciousStone3.png"),
                            ModAPI.LoadSprite("Textures/PreciousStone4.png"),
                            ModAPI.LoadSprite("Textures/PreciousStone5.png")
                        };

                        Instance.FixColliders();
                    }
                }
            );
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Stick"),
                    NameOverride = "Suitcase" +tag,
                    NameToOrderByOverride = "!",
                    DescriptionOverride = "A suitcase. Made of leather.",
                    CategoryOverride = ModAPI.FindCategory("Misc."),
                    ThumbnailOverride = ModAPI.LoadSprite("Textures/SuitcaseThumb.png"),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Textures/Suitcase1.png");

                        Instance.GetOrAddComponent<RandomSpriteBehaviour>().sprites = new Sprite[]
                        {
                            ModAPI.LoadSprite("Textures/Suitcase1.png"),
                            ModAPI.LoadSprite("Textures/Suitcase2.png")
                        };

                        var phys = Instance.GetComponent<PhysicalBehaviour>();
                        phys.TrueInitialMass = 0.06f;
                        phys.InitialMass = 0.06f;
                        phys.rigidbody.mass = 0.06f;
                        var properties = ModAPI.FindPhysicalProperties("Bowling pin");
                        properties.name = "Suitcase";
                        properties.Buoyancy = 0.05f;
                        properties.Flammability = 25f;
                        properties.Brittleness = 40f;
                        properties.BulletSpeedAbsorptionPower = 0.01f;
                        properties.Burnrate = 1f;
                        properties.Softness = 0.6f;
                        properties.SoftImpact = new AudioClip[]
                        {
                            ModAPI.LoadSound("Sounds/Suitcase1.mp3"),
                            ModAPI.LoadSound("Sounds/Suitcase2.mp3")
                        };
                        properties.HardImpact = new AudioClip[]
                        {
                            ModAPI.LoadSound("Sounds/Suitcase1.mp3"),
                            ModAPI.LoadSound("Sounds/Suitcase2.mp3")
                        };
                        properties.StabSound = new AudioClip[]
                        {
                            ModAPI.LoadSound("Sounds/Suitcase1.mp3"),
                            ModAPI.LoadSound("Sounds/Suitcase2.mp3")
                        };
                        phys.Properties = properties;

                        Instance.FixColliders();
                    }
                }
            );
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Rod"),
                    NameOverride = "Trophy" +tag,
                    NameToOrderByOverride = "!",
                    DescriptionOverride = "Awarded to the winners in the competition.",
                    CategoryOverride = ModAPI.FindCategory("Misc."),
                    ThumbnailOverride = ModAPI.LoadSprite("Textures/TrophyThumb.png"),
                    AfterSpawn = (Instance) =>
                    {
                        GameObject.Destroy(Instance.GetComponent<GlowingHotMetalBehaviour>());

                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Textures/Trophy1.png");

                        Instance.GetOrAddComponent<RandomSpriteBehaviour>().sprites = new Sprite[]
                        {
                            ModAPI.LoadSprite("Textures/Trophy1.png"),
                            ModAPI.LoadSprite("Textures/Trophy2.png"),
                            ModAPI.LoadSprite("Textures/Trophy3.png")
                        };

                        var phys = Instance.GetComponent<PhysicalBehaviour>();
                        phys.TrueInitialMass = 0.075f;
                        phys.InitialMass = 0.075f;
                        phys.rigidbody.mass = 0.075f;
                        phys.Properties = ModAPI.FindPhysicalProperties("Metal");
                        phys.HoldingPositions = new Vector3[]
                        {
                            new Vector3(-0.3f, 0.17f),
                            new Vector3(0.299f, 0.17f)
                        };

                        Instance.FixColliders();
                    }
                }
            );
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Stick"),
                    NameOverride = "First Aid Kit" +tag,
                    NameToOrderByOverride = "!",
                    DescriptionOverride = "A first aid kit.",
                    CategoryOverride = ModAPI.FindCategory("Misc."),
                    ThumbnailOverride = ModAPI.LoadSprite("Textures/FirstAidKitThumb.png"),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Textures/FirstAidKit1.png");

                        Instance.GetOrAddComponent<RandomSpriteBehaviour>().sprites = new Sprite[]
                        {
                            ModAPI.LoadSprite("Textures/FirstAidKit1.png"),
                            ModAPI.LoadSprite("Textures/FirstAidKit2.png")
                        };

                        var phys = Instance.GetComponent<PhysicalBehaviour>();
                        phys.TrueInitialMass = 0.04f;
                        phys.InitialMass = 0.04f;
                        phys.rigidbody.mass = 0.04f;
                        var properties = ModAPI.FindPhysicalProperties("Bowling pin");
                        properties.name = "First Aid Kit";
                        properties.Buoyancy = 0.15f;
                        properties.Flammability = 25f;
                        properties.Brittleness = 20f;
                        properties.BulletSpeedAbsorptionPower = 0.02f;
                        properties.Burnrate = 0.8f;
                        properties.Softness = 0.55f;
                        properties.SoftImpact = new AudioClip[]
                        {
                            ModAPI.LoadSound("Sounds/FirstAidKit1.mp3"),
                            ModAPI.LoadSound("Sounds/FirstAidKit2.mp3"),
                            ModAPI.LoadSound("Sounds/FirstAidKit3.mp3")
                        };
                        properties.HardImpact = new AudioClip[]
                        {
                            ModAPI.LoadSound("Sounds/FirstAidKit1.mp3"),
                            ModAPI.LoadSound("Sounds/FirstAidKit2.mp3"),
                            ModAPI.LoadSound("Sounds/FirstAidKit3.mp3")
                        };
                        properties.StabSound = new AudioClip[]
                        {
                            ModAPI.LoadSound("Sounds/FirstAidKit1.mp3"),
                            ModAPI.LoadSound("Sounds/FirstAidKit2.mp3"),
                            ModAPI.LoadSound("Sounds/FirstAidKit3.mp3")
                        };
                        phys.Properties = properties;

                        Instance.FixColliders();
                    }
                }
            );
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Plate"),
                    NameOverride = "Jug" +tag,
                    NameToOrderByOverride = "!",
                    DescriptionOverride = "Destructible jug.",
                    CategoryOverride = ModAPI.FindCategory("Misc."),
                    ThumbnailOverride = ModAPI.LoadSprite("Textures/JugThumb.png"),
                    AfterSpawn = (Instance) =>
                    {
                        GameObject.Destroy(Instance.GetComponent<BoxCollider2D>());
                        GameObject.Destroy(Instance.GetComponent<DestroyableBehaviour>());

                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Textures/Jug.png");

                        if(Instance.transform.localScale == new Vector3(1f, 1f, 1f))
                        {
                            Instance.transform.localScale = new Vector3(0.8f, 0.8f, 1f);
                        }
                        if(Instance.transform.localScale == new Vector3(-1f, 1f, 1f))
                        {
                            Instance.transform.localScale = new Vector3(-0.8f, 0.8f, 1f);
                        }

                        var phys = Instance.GetComponent<PhysicalBehaviour>();
                        phys.TrueInitialMass = 0.06f;
                        phys.InitialMass = 0.06f;
                        phys.rigidbody.mass = 0.06f;
                        phys.HoldingPositions = new Vector3[]
                        {
                            new Vector3(0.232f, 0.009f),
                            new Vector3(0.229f, 0.071f)
                        };

                        var destructibleObjectBehaviour = Instance.GetOrAddComponent<DestructibleObjectBehaviour>();
                        destructibleObjectBehaviour.breakSounds = new AudioClip[]
                        {
                            ModAPI.LoadSound("Sounds/JugBreak1.mp3"),
                            ModAPI.LoadSound("Sounds/JugBreak2.mp3")
                        };
                        destructibleObjectBehaviour.debrisSprites = new Sprite[]
                        {
                            ModAPI.LoadSprite("Textures/JugDebris1.png"),
                            ModAPI.LoadSprite("Textures/JugDebris2.png"),
                            ModAPI.LoadSprite("Textures/JugDebris3.png"),
                            ModAPI.LoadSprite("Textures/JugDebris4.png"),
                            ModAPI.LoadSprite("Textures/JugDebris5.png"),
                            ModAPI.LoadSprite("Textures/JugDebris6.png"),
                            ModAPI.LoadSprite("Textures/JugDebris7.png"),
                            ModAPI.LoadSprite("Textures/JugDebris8.png"),
                            ModAPI.LoadSprite("Textures/JugDebris9.png")
                        };
                        destructibleObjectBehaviour.debrisPositions = new Vector3[]
                        {
                            new Vector3(0f, 0f),
                            new Vector3(0f, 0f),
                            new Vector3(0f, 0f),
                            new Vector3(0f, 0f),
                            new Vector3(0f, 0f),
                            new Vector3(0f, 0f),
                            new Vector3(0f, 0f),
                            new Vector3(0f, 0f),
                            new Vector3(0f, 0f)
                        };
                        destructibleObjectBehaviour.debrisProperties = ModAPI.FindPhysicalProperties("Glass");
                        destructibleObjectBehaviour.debrisMass = 0.0035f;
                        destructibleObjectBehaviour.BreakingThreshold = 0.1f;
                        destructibleObjectBehaviour.debrisNumber = 9;

                        Instance.AddComponent<BoxCollider2D>();
                    }
                }
            );
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Metal Pole"),
                    NameOverride = "Metal Barrel" +tag,
                    NameToOrderByOverride = "!",
                    DescriptionOverride = "Barrel made of metal.",
                    CategoryOverride = ModAPI.FindCategory("Misc."),
                    ThumbnailOverride = ModAPI.LoadSprite("Textures/MetalBarrelThumb.png"),
                    AfterSpawn = (Instance) =>
                    {
                        GameObject.Destroy(Instance.GetComponent<GlowingHotMetalBehaviour>());

                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Textures/MetalBarrel1.png");

                        Instance.GetOrAddComponent<RandomSpriteBehaviour>().sprites = new Sprite[]
                        {
                            ModAPI.LoadSprite("Textures/MetalBarrel1.png"),
                            ModAPI.LoadSprite("Textures/MetalBarrel2.png"),
                            ModAPI.LoadSprite("Textures/MetalBarrel3.png"),
                            ModAPI.LoadSprite("Textures/MetalBarrel4.png"),
                            ModAPI.LoadSprite("Textures/MetalBarrel5.png")
                        };

                        var phys = Instance.GetComponent<PhysicalBehaviour>();
                        phys.TrueInitialMass = 0.85f;
                        phys.InitialMass = 0.85f;
                        phys.rigidbody.mass = 0.85f;
                        phys.Properties = ModAPI.FindPhysicalProperties("Flammable metal");

                        Instance.FixColliders();
                    }
                }
            );
        }
    }
    public class DestructibleObjectBehaviour : MonoBehaviour
    {
        private void OnCollisionStay2D(Collision2D collision)
        {
            EvaluateCollision(collision);
        }
        private void OnCollisionEnter2D(Collision2D collision)
        {
            EvaluateCollision(collision);
        }
        private void EvaluateCollision(Collision2D collision)
        {
            int contacts = collision.GetContacts(contactBuffer);
            float num = Utils.GetMinImpulse(contactBuffer, contacts);
            Vector2 point = contactBuffer[0].point;
            if(num < BreakingThreshold)
            {
                return;
            }
            Broken = true;
            Break();
        }
        private void Shot(Shot shot)
        {
            if(ShotDamage)
            {
                Break();
            }
        }
        private void OnFragmentHit()
        {
            Break();
        }
        private void Break()
        {
            GameObject audioObject = new GameObject("Audio Object");
            audioObject.transform.position = gameObject.transform.position;
            AudioSource audioSource = audioObject.AddComponent<AudioSource>();
            audioSource.spread = 0f;
            audioSource.volume = 0.8f;
            audioSource.minDistance = 10f;
            audioSource.spatialBlend = 1f;
            audioSource.dopplerLevel = 1f;
            audioSource.PlayOneShot(breakSounds.PickRandom(), 1f);
            audioObject.AddComponent<AudioSourceTimeScaleBehaviour>();
            for(int i = 0; i < debrisNumber; i++)
            {
                debris = ModAPI.CreatePhysicalObject("Debris "+i.ToString(), debrisSprites[i]);
                debris.transform.localScale = gameObject.transform.localScale;
                debris.transform.position = gameObject.transform.position + debrisPositions[i];
                debris.GetComponent<Rigidbody2D>().velocity = gameObject.GetComponent<Rigidbody2D>().velocity + UnityEngine.Random.insideUnitCircle * 2f;
                debris.GetComponent<Rigidbody2D>().angularVelocity = gameObject.GetComponent<Rigidbody2D>().angularVelocity + UnityEngine.Random.Range(-200, 200);
                debris.GetOrAddComponent<DebrisComponent>();
                var debrisPhys = debris.GetComponent<PhysicalBehaviour>();
                debrisPhys.Properties = debrisProperties;
                debrisPhys.InitialMass = debrisMass;
                debrisPhys.rigidbody.mass = debrisMass;
                debrisPhys.TrueInitialMass = debrisPhys.InitialMass;
                debris.AddComponent<BoxCollider2D>();
                debris.layer = 10;
                debris.AddComponent<ActOnDestroy>().DestroyWith = new GameObject[]
                {
                    audioObject
                };
            }
            gameObject.SetActive(false);
        }

        public bool Broken = false;
        public bool ShotDamage = true;
        public GameObject debris;
        private ContactPoint2D[] contactBuffer = new ContactPoint2D[12];
        public AudioClip[] breakSounds;
        public Sprite[] debrisSprites;
        public Vector3[] debrisPositions;
        public PhysicalProperties debrisProperties;
        public float debrisMass;
        public float BreakingThreshold;
        public int debrisNumber;
    }
    public class BottleBehaviour : MonoBehaviour
    {
        private void Start()
        {
            randomSprites = gameObject.GetComponent<RandomSpriteBehaviour>();
            destructibleObjectBehaviour = gameObject.GetComponent<DestructibleObjectBehaviour>();
        }
        private void Update()
        {
            if(randomSprites.chosenIndex == 0)
            {
                destructibleObjectBehaviour.debrisSprites = sprites1;
            }
            if(randomSprites.chosenIndex == 1)
            {
                destructibleObjectBehaviour.debrisSprites = sprites2;
            }
            if(randomSprites.chosenIndex == 2)
            {
                destructibleObjectBehaviour.debrisSprites = sprites3;
            }
        }

        private RandomSpriteBehaviour randomSprites;
        private DestructibleObjectBehaviour destructibleObjectBehaviour;
        public Sprite[] sprites1;
        public Sprite[] sprites2;
        public Sprite[] sprites3;
    }
    public class SafeBehaviour : MonoBehaviour
    {
        private void Start()
        {
            renderer = gameObject.GetComponent<SpriteRenderer>();
            audioSource = gameObject.AddComponent<AudioSource>();
            audioSource.spread = 0f;
            audioSource.volume = 1f;
            audioSource.minDistance = 10f;
            audioSource.spatialBlend = 1f;
            audioSource.dopplerLevel = 1f;
            gameObject.AddComponent<AudioSourceTimeScaleBehaviour>();
            if(chosenIndex == -1)
            {
                chosenIndex = UnityEngine.Random.Range(0, Textures.Length);
            }
            UpdateActivation();
        }
        private void Use()
        {
            Activated = !Activated;
            if(Activated)
            {
                audioSource.PlayOneShot(OpenSound, 1f);
            }
            else
            {
                audioSource.PlayOneShot(ClosedSound, 1f);
            }
            UpdateActivation();
        }
        private void UpdateActivation()
        {
            SafeBehaviour.SafeSprites safeSprites = Textures[chosenIndex];
            if(Activated)
            {
                renderer.sprite = safeSprites.OpenSprite;
                Collider.isTrigger = true;
            }
            else
            {
                renderer.sprite = safeSprites.ClosedSprite;
                Collider.isTrigger = false;
            }
            gameObject.GetComponent<PhysicalBehaviour>().RefreshOutline();
        }

        public bool Activated;
        private SpriteRenderer renderer;
        private AudioSource audioSource;
        public SafeBehaviour.SafeSprites[] Textures;
        public Collider2D Collider;
        public AudioClip OpenSound;
        public AudioClip ClosedSound;
        public int chosenIndex = -1;
        public class SafeSprites
        {
            public Sprite OpenSprite;
            public Sprite ClosedSprite;
        }
    }
    public class ClockBehaviour : MonoBehaviour
    {
        private void Start()
        {
            audioSource = gameObject.AddComponent<AudioSource>();
            audioSource.spread = 0f;
            audioSource.volume = 1f;
            audioSource.minDistance = 10f;
            audioSource.spatialBlend = 1f;
            audioSource.dopplerLevel = 1f;
            gameObject.AddComponent<AudioSourceTimeScaleBehaviour>();
        }
        private void Use()
        {
            Activated = !Activated;
        }
        private void FixedUpdate()
        {
            if(Activated)
            {
                TimeSpan timespan = DateTime.Now.TimeOfDay;
                hourArrow.localRotation = Quaternion.Euler(0f, 0f, (float)timespan.TotalHours * -hoursToDegrees);
                minuteArrow.localRotation = Quaternion.Euler(0f, 0f, (float)timespan.TotalMinutes * -minutesToDegrees);
                secondArrow.localRotation = Quaternion.Euler(0f, 0f, (float)timespan.TotalSeconds * -secondsToDegrees);
                if(!TickAudio)
                {
                    StartCoroutine(Tick());
                }
            }
            else
            {
                if(TickAudio)
                {
                    StopCoroutine(Tick());
                    TickAudio = false;
                }
            }
        }
        private IEnumerator Tick()
        {
            TickAudio = true;
            while(Activated)
            {
                yield return new WaitForSecondsRealtime(1f);
                audioSource.PlayOneShot(tick, 1f);
                yield return null;
            }
        }

        public bool Activated = true;
        private bool TickAudio;
        private AudioSource audioSource;
        public Transform hourArrow;
        public Transform minuteArrow;
        public Transform secondArrow;
        public AudioClip tick;
        private const float hoursToDegrees = 360f / 12f;
        private const float minutesToDegrees = 360f / 60f;
        private const float secondsToDegrees = 360f / 60f;
    }
}